@@ -2,6 +2,8 @@ import collections
 import copy
 import new
 
+from operator import itemgetter
+
 
 class EnumConstructionException(Exception):
     """
@@ -28,12 +30,16 @@ def enum(**enums):
         >>> MY_ENUM.BAR
         2
     """
-    # NB: cheating here by just maintaining a copy of the original dict for iteration because iterators are hard
-    #  it must be a deepcopy because new.classobj() modifies the original
+    # Enum values must be hashable to support reverse lookup.
+    if not all(isinstance(val, collections.Hashable) for val in enums.itervalues()):
+        raise EnumConstructionException('All enum values must be hashable.')
+
+    # Cheating by maintaining a copy of original dict for iteration b/c iterators are hard.
+    # It must be a deepcopy because new.classobj() modifies the original.
     en = copy.deepcopy(enums)
     e = new.classobj('Enum', (), enums)
     e._dict = en
-    e.choices = [(v, k) for k, v in en.iteritems()]
+    e.choices = [(v, k) for k, v in sorted(en.iteritems(), key=itemgetter(1))]  # DEPRECATED
     e.get_id_by_label = e._dict.get
     e.get_label_by_id = dict([(v, k) for (k, v) in e._dict.items()]).get
 
@@ -41,8 +47,6 @@ def enum(**enums):
 
 
 class RichEnumValue(object):
-    """
-    """
     def __init__(self, canonical_name, display_name, *args, **kwargs):
         self.canonical_name = canonical_name
         self.display_name = display_name
@@ -76,6 +80,8 @@ class RichEnumValue(object):
 
     def choicify(self, value_field="canonical_name", display_field="display_name"):
         """
+        DEPRECATED
+
         Returns a tuple that's compatible with Django's choices.
         https://docs.djangoproject.com/en/dev/ref/models/fields/#choices
         """
@@ -133,7 +139,8 @@ def _setup_members(cls_attrs, cls_parents, member_cls):
                 raise EnumConstructionException("Invalid attribute: %s" % attr_key)
 
         if cls_parents not in [(object, ), (_EnumMethods, )] and not members:
-            raise EnumConstructionException("Must specify at least one attribute when using RichEnum")
+            raise EnumConstructionException(
+                "Must specify at least one attribute when using RichEnum")
 
         return members
 
@@ -213,17 +220,22 @@ class _EnumMethods(object):
     @classmethod
     def choices(cls, value_field='canonical_name', display_field='display_name'):
         """
+        DEPRECATED
+
         Returns a list of 2-tuples to be used as an argument to Django Field.choices
 
         Implementation note: choices() can't be a property
-                             See: http://www.no-ack.org/2011/03/strange-behavior-with-properties-on.html and http://utcc.utoronto.ca/~cks/space/blog/python/UsingMetaclass03
+        See:
+            http://www.no-ack.org/2011/03/strange-behavior-with-properties-on.html
+            http://utcc.utoronto.ca/~cks/space/blog/python/UsingMetaclass03
         """
-        return [m.choicify(value_field=value_field, display_field=display_field) for m in cls._MEMBERS]  # pylint: disable=E1101
+        return [m.choicify(value_field=value_field, display_field=display_field) for m in cls.members()]
 
 
 class RichEnum(_EnumMethods):
     """
-    Enumeration that can represent a name for referencing (canonical_name) and a name for displaying (display_name).
+    Enumeration that can represent a name for referencing (canonical_name) and
+    a name for displaying (display_name).
 
     Usage:
 
@@ -238,18 +250,23 @@ class RichEnum(_EnumMethods):
 
     Notes:
         1) display_name can be a string that's marked for translation.
-           We recommend that display name be a lazily translated string for RichEnums in constants files.
-        2) Subclassing RichEnumValue is nice, that way when the RichEnumValue is logged/printed, it'll show your custom RichEnumValue and it'll be easier to differentiate between all of your different RichEnums.
-    """
+           We recommend that display name be a lazily translated string for
+           RichEnums in constants files.
+        2) Subclassing RichEnumValue is nice, that way when the RichEnumValue
+           is logged/printed, it'll show your custom RichEnumValue and it'll be
+           easier to differentiate between all of your different RichEnums.
+
+   """
     __metaclass__ = _RichEnumMetaclass
 
 
 class OrderedRichEnum(_EnumMethods):
     """
     Use OrderedRichEnum when you need a RichEnum with index-based
-    access into the enum, e.g. OrderedRichEnumExample.from_index(0).
+    access into the enum, e.g. OrderedRichEnumExample.from_index(0),
+    and iteration over the enums sorted by index.
 
-    If you don't need legacy index-based access, please use regular RichEnum
+    If you don't need index-based access or sorted entries, please use regular RichEnum
     instead. It's more explicit.
 
     Usage:
@@ -268,5 +285,5 @@ class OrderedRichEnum(_EnumMethods):
     __metaclass__ = _OrderedRichEnumMetaclass
 
     @classmethod
-    def from_index(cls, index):  # Purely for consistency
+    def from_index(cls, index):
         return cls.lookup('index', index)
